package solutions.bitbadger.documents.groovy.query

import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
//import solutions.bitbadger.documents.DocumentException
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.query.CountQuery
import solutions.bitbadger.documents.support.ForceDialect

import static solutions.bitbadger.documents.support.TypesKt.TEST_TABLE
import static groovy.test.GroovyAssert.*

/**
 * Unit tests for the `Count` object
 */
@DisplayName('JVM | Groovy | Query | CountQuery')
class CountQueryTest {

    /**
     * Clear the connection string (resets Dialect)
     */
    @AfterEach
    void cleanUp() {
        ForceDialect.none()
    }

    @Test
    @DisplayName('all generates correctly')
    void all() {
        assertEquals('Count query not constructed correctly', "SELECT COUNT(*) AS it FROM $TEST_TABLE".toString(),
                CountQuery.all(TEST_TABLE))
    }

    @Test
    @DisplayName('byFields generates correctly | PostgreSQL')
    void byFieldsPostgres() {
        ForceDialect.postgres()
        assertEquals('Count query not constructed correctly',
                "SELECT COUNT(*) AS it FROM $TEST_TABLE WHERE data->>'test' = :field0".toString(),
                CountQuery.byFields(TEST_TABLE, List.of(Field.equal('test', '', ':field0'))))
    }

    @Test
    @DisplayName('byFields generates correctly | SQLite')
    void byFieldsSQLite() {
        ForceDialect.sqlite()
        assertEquals('Count query not constructed correctly',
                "SELECT COUNT(*) AS it FROM $TEST_TABLE WHERE data->>'test' = :field0".toString(),
                CountQuery.byFields(TEST_TABLE, List.of(Field.equal('test', '', ':field0'))))
    }

    @Test
    @DisplayName('byContains generates correctly | PostgreSQL')
    void byContainsPostgres() {
        ForceDialect.postgres()
        assertEquals('Count query not constructed correctly',
                "SELECT COUNT(*) AS it FROM $TEST_TABLE WHERE data @> :criteria".toString(),
                CountQuery.byContains(TEST_TABLE))
    }

//    TODO: resolve java.base open issue
//    @Test
//    @DisplayName('byContains fails | SQLite')
//    void byContainsSQLite() {
//        ForceDialect.sqlite()
//        assertThrows(DocumentException) { CountQuery.byContains(TEST_TABLE) }
//    }

    @Test
    @DisplayName('byJsonPath generates correctly | PostgreSQL')
    void byJsonPathPostgres() {
        ForceDialect.postgres()
        assertEquals('Count query not constructed correctly',
                "SELECT COUNT(*) AS it FROM $TEST_TABLE WHERE jsonb_path_exists(data, :path::jsonpath)".toString(),
                CountQuery.byJsonPath(TEST_TABLE))
    }

//    TODO: resolve java.base open issue
//    @Test
//    @DisplayName('byJsonPath fails | SQLite')
//    void byJsonPathSQLite() {
//        ForceDialect.sqlite()
//        assertThrows(DocumentException) { CountQuery.byJsonPath(TEST_TABLE) }
//    }
}
