package solutions.bitbadger.documents.groovy.query

import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
//import solutions.bitbadger.documents.DocumentException
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.query.DeleteQuery
import solutions.bitbadger.documents.support.ForceDialect

import static solutions.bitbadger.documents.support.TypesKt.TEST_TABLE
import static groovy.test.GroovyAssert.*

/**
 * Unit tests for the `Delete` object
 */
@DisplayName('JVM | Groovy | Query | DeleteQuery')
class DeleteQueryTest {

    /**
     * Clear the connection string (resets Dialect)
     */
    @AfterEach
    void cleanUp() {
        ForceDialect.none()
    }

    @Test
    @DisplayName('byId generates correctly | PostgreSQL')
    void byIdPostgres() {
        ForceDialect.postgres()
        assertEquals('Delete query not constructed correctly',
                "DELETE FROM $TEST_TABLE WHERE data->>'id' = :id".toString(), DeleteQuery.byId(TEST_TABLE))
    }

    @Test
    @DisplayName('byId generates correctly | SQLite')
    void byIdSQLite() {
        ForceDialect.sqlite()
        assertEquals('Delete query not constructed correctly',
                "DELETE FROM $TEST_TABLE WHERE data->>'id' = :id".toString(), DeleteQuery.byId(TEST_TABLE))
    }

    @Test
    @DisplayName('byFields generates correctly | PostgreSQL')
    void byFieldsPostgres() {
        ForceDialect.postgres()
        assertEquals('Delete query not constructed correctly',
                "DELETE FROM $TEST_TABLE WHERE data->>'a' = :b".toString(),
                DeleteQuery.byFields(TEST_TABLE, List.of(Field.equal('a', '', ':b'))))
    }

    @Test
    @DisplayName('byFields generates correctly | SQLite')
    void byFieldsSQLite() {
        ForceDialect.sqlite()
        assertEquals('Delete query not constructed correctly',
                "DELETE FROM $TEST_TABLE WHERE data->>'a' = :b".toString(),
                DeleteQuery.byFields(TEST_TABLE, List.of(Field.equal('a', '', ':b'))))
    }

    @Test
    @DisplayName('byContains generates correctly | PostgreSQL')
    void byContainsPostgres() {
        ForceDialect.postgres()
        assertEquals('Delete query not constructed correctly',
                "DELETE FROM $TEST_TABLE WHERE data @> :criteria".toString(), DeleteQuery.byContains(TEST_TABLE))
    }

//    TODO: resolve java.base open issue
//    @Test
//    @DisplayName('byContains fails | SQLite')
//    void byContainsSQLite() {
//        ForceDialect.sqlite()
//        assertThrows(DocumentException) { DeleteQuery.byContains(TEST_TABLE) }
//    }

    @Test
    @DisplayName('byJsonPath generates correctly | PostgreSQL')
    void byJsonPathPostgres() {
        ForceDialect.postgres()
        assertEquals('Delete query not constructed correctly',
                "DELETE FROM $TEST_TABLE WHERE jsonb_path_exists(data, :path::jsonpath)".toString(),
                DeleteQuery.byJsonPath(TEST_TABLE))
    }

//    TODO: resolve java.base open issue
//    @Test
//    @DisplayName('byJsonPath fails | SQLite')
//    void byJsonPathSQLite() {
//        ForceDialect.sqlite()
//        assertThrows(DocumentException) { DeleteQuery.byJsonPath(TEST_TABLE) }
//    }
}
